#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
usage: python EnrichmentAnalysisForSingleTrans.py -i <all_ratio.txt> -o <output_prefix> -c <coorFile> -s transcript_id --id-type transcript_id --unit codon [-S| --ymin|--ymax...]
'''

from .FunctionDefinition import *
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages


def plot_ratio_for_single_trans(ratio,trans,inOutPrefix,ymin,ymax,unit,axvline,axhline,color='b',text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the ratio dsitribution'''
	plt.rc("font",weight="bold")
	winLen=len(ratio)
	with PdfPages(inOutPrefix + "_enrichment_ratio.pdf") as pdf:
		fig=plt.figure(figsize=(16,8))
		ax=fig.add_subplot(111)
		plt.plot(np.arange(0,winLen),ratio,color=color,label=trans,linewidth=1)
		if axvline:
			ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
		else:
			pass

		ax.set_xlabel("Distance from start codon" + "("+unit+")",fontdict=text_font)
		ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
		if axhline:
			ax.axhline(1,color="r",dashes=[2,3],clip_on=False,linewidth=2)
		else:
			pass
		ax.spines["top"].set_visible(False)
		ax.spines["right"].set_visible(False)
		ax.spines["bottom"].set_linewidth(2)
		ax.spines["left"].set_linewidth(2)
		ax.tick_params(which="both",width=2,labelsize=20)
		if not ymin and not ymax:
			pass
		elif not ymin and ymax:
			ax.set_ylim(0,ymax)
		elif ymin and not ymax:
			raise IOError("Please offer the ymax parameter as well!")
		elif ymin and ymax:
			ax.set_ylim(ymin,ymax)
		else:
			raise IOError("Please enter correct ymin and ymax parameters!")
		plt.legend(loc="best",prop=legend_font)
		plt.tight_layout()
		pdf.savefig(fig)
		plt.close()

def plot_ratio_for_all_trans(ratio_dict,transList,inOutPrefix,ymin,ymax,unit,axvline,axhline,start,window,step,slideWindow,color='b',text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the ratio dsitribution'''
	plt.rc("font",weight="bold")
	with PdfPages(inOutPrefix + "_enrichment_ratio.pdf") as pdf:
		for trans in transList:
			ratio=ratio_dict[trans]
			if slideWindow:
				ratio=slide_window_average(ratio,start,window,step)
			else:
				pass
			winLen=len(ratio)
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			plt.plot(np.arange(0,winLen),ratio,color=color,label=trans,linewidth=1)
			if axvline:
				ax.axvline(axvline,color="r",dashes=[1,2],clip_on=False,linewidth=2)
			else:
				pass

			ax.set_xlabel("Distance from start codon" + "("+unit+")",fontdict=text_font)
			ax.set_ylabel("Mean enrichment (A.U)",fontdict=text_font)
			if axhline:
				ax.axhline(1,color="r",dashes=[2,3],clip_on=False,linewidth=2)
			else:
				pass
			ax.spines["top"].set_visible(False)
			ax.spines["right"].set_visible(False)
			ax.spines["bottom"].set_linewidth(2)
			ax.spines["left"].set_linewidth(2)
			ax.tick_params(which="both",width=2,labelsize=20)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.legend(loc="best",prop=legend_font)
			plt.tight_layout()
			pdf.savefig(fig)
			plt.close()

def get_density_dict(densityFile):
	density_dict={}
	with open(densityFile,'r') as f:
		i=0
		for line in f:
			i+=1
			if i==1:
				continue
			tmp=line.strip().split("\t")
			trans_id=tmp[0]
			density_dict[trans_id]=[float(i) for i in tmp[1:]]
	return density_dict


def ID_transformation(transcript,coorFile,Type='singleTrans',id_type="transcript_id"):
	transID2geneID,transID2geneName=reload_transcripts_information(coorFile)[4:6]
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if Type=='transList':
		if id_type == 'transcript_id':
			select_trans=set(transcript)
		elif id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in transcript if gene_id in geneID2transID]
			select_trans=set(tmp)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif id_type == 'gene_name' or id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in transcript if gene_name in geneName2transID]
			select_trans=set(tmp)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	elif Type == 'singleTrans':
		if id_type == 'transcript_id':
			select_trans=transcript
		elif id_type == 'gene_id':
			select_trans=geneID2transID[transcript]
		elif id_type == 'gene_name' or id_type=='gene_symbol':
			select_trans=geneName2transID[transcript]
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		pass
	return select_trans

def slide_window_average(ratio,start,window,step):
	''' Used for calculating mean density with a slide window'''

	winLen=len(ratio)
	tmp_data=np.zeros(winLen)
	tmp_data[0:int(start)]+=ratio[0:int(start)]
	tmp_data[-int(start):]+=ratio[-int(start):]
	for j in np.arange(start,winLen-start,step):
		tmp_data[j]+=np.mean(ratio[(j-int((window-1)/2)):(j+int((window-1)/2))])
	return tmp_data

def main():
	parser=create_parser_for_single_ratio_plot()
	(options,args)=parser.parse_args()
	if not options.ratioFile or not options.output_prefix or not options.coorFile:
		raise IOError("Please reset your parameters!")
	if options.window%2 == 0:
		raise IOError("Please reset your --window parameter. It must be a odd number.")
	if (options.start_position-1) < (options.window-1)/2:
		raise IOError("Please reset your --step and --window parameters. The (window-1)/2 must be less than start-1")
	ratio_dict=get_density_dict(options.ratioFile)
	text_font={"size":40,"family":"Arial","weight":"bold"}
	legend_font={"size":30,"family":"Arial","weight":"bold"}
	if options.singleTrans and not options.in_selectTrans:
		select_trans=ID_transformation(options.singleTrans,options.coorFile,Type='singleTrans',id_type=options.id_type)
		if select_trans not in ratio_dict.keys():
			raise IOError("Please reset your -s parameter! "+options.singleTrans+" not in " + options.ratioFile+"!")
		ratio=ratio_dict[select_trans]
		if not options.slideWindow:
			pass
		else:
			ratio=slide_window_average(ratio,options.start_position,options.window,options.step)
		plot_ratio_for_single_trans(ratio,options.singleTrans+":"+select_trans,options.output_prefix,options.ymin,options.ymax,options.unit,options.axvline,options.axhline,color='b',text_font=text_font,legend_font=legend_font)
		print("Finish the step of ratio plot!",file=sys.stderr)
	elif options.in_selectTrans and not options.singleTrans:
		transList=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=ID_transformation(transList,options.coorFile,Type='transList',id_type=options.id_type)
		select_trans=select_trans.intersection(set(ratio_dict.keys()))
		plot_ratio_for_all_trans(ratio_dict,select_trans,options.output_prefix,options.ymin,options.ymax,options.unit,options.axvline,options.axhline,options.start_position,options.window,options.step,options.slideWindow,color='b',text_font=text_font,legend_font=legend_font)
		print("Finish the step of ratio plot!",file=sys.stderr)
	elif options.in_selectTrans and options.singleTrans:
		raise IOError("The -s and -S are mutually exclusive!")
	else:
		plot_ratio_for_all_trans(ratio_dict,ratio_dict.keys(),options.output_prefix,options.ymin,options.ymax,options.unit,options.axvline,options.axhline,options.start_position,options.window,options.step,options.slideWindow,color='b',text_font=text_font,legend_font=legend_font)
		print("Finish the step of ratio plot!",file=sys.stderr)

if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Description: The script is used for hydropathy or charge calculaiton for a specific region
input:
1) cds sequences with fasta format you are interested in. Different fasta files would be separated by comma.
2) hydropathy index file download from AAindex. Three columns: AA(single character), amino acids (three characters), and hydropathy, just lile this:
AA	amio_acids	hydropathy
A	Ala 	1.8
R	Arg 	-4.5
N	Asn 	-3.5
D	Asp 	-3.5
charge index file is just like:
AA	amio_acids	charges
A	Ala 	0
R	Arg 	1
N	Asn 	0
D	Asp 	-1
output:
1) hydropathy or charge values at each position along transcripts. One fasta file, one hydropathy or charge value file.
2) a meta results in a specific regions.
'''

from .FunctionDefinition import *
import re
from itertools import groupby




def get_hydropathy_or_charge_vector(index_dict,AA_seq):
	vector=np.zeros(len(AA_seq),dtype="float")
	for i in np.arange(len(AA_seq)):
		vector[i]+=index_dict[AA_seq[i]]
	return vector


def hydropathy_or_charge(index_dict,transcriptFile,in_extendRegionLengthParma,in_regionLengthParma,table):
	trans_seq_dict=fastaIter(transcriptFile)
	in_selectTrans=set(trans_seq_dict.keys())
	passTransSet=set()
	valuePerCodon={}
	startDensity=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stopDensity =np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	startNormedWindowsList=[]
	startWindowsExistPosList=[]
	stopNormedWindowsList=[]
	stopWindowsExistPosList=[]
	for trans in in_selectTrans:
		cds_seq=trans_seq_dict[trans]
		AA_seq=translation(cds_seq,table=table,cds=False)
		vector=get_hydropathy_or_charge_vector(index_dict,AA_seq)
		###get trans density vector
		(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,vector,0) #start codon coor is 0 (0-based)
		(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,vector,(len(vector)-1)) #stop codon coor is len-1 (0-based)
		startNormedWindowsList.append(tmpStartWin)
		startWindowsExistPosList.append(tmpStartPos)
		stopNormedWindowsList.append(tmpStopWin)
		stopWindowsExistPosList.append(tmpStopPos)
		passTransSet.add(trans)
		valuePerCodon[trans]=vector
	###norm per position by mean or median
	startNormedWindowsList=np.array(startNormedWindowsList)
	startWindowsExistPosList=np.array(startWindowsExistPosList)
	stopNormedWindowsList=np.array(stopNormedWindowsList)
	stopWindowsExistPosList=np.array(stopWindowsExistPosList)
	#
	for terms in range(in_extendRegionLengthParma+in_regionLengthParma+1):
		startDensity[terms]=np.mean(startNormedWindowsList[np.where(startWindowsExistPosList[:,terms]==1),terms])
		stopDensity[terms] =np.mean(stopNormedWindowsList[np.where(stopWindowsExistPosList[:,terms]==1),terms])
	return startDensity,stopDensity,valuePerCodon


def write_hydropathy_or_charge_dataframe(inFastaAttr,outFile):
	data=[]
	for fasta in inFastaAttr:
		k=pd.DataFrame([fasta.fastaLegend]*len(fasta.startDensity))
		start=pd.DataFrame(fasta.startDensity)
		stop=pd.DataFrame(fasta.stopDensity)
		values=pd.merge(start,stop,how="left",left_index=True,right_index=True)
		values=pd.merge(k,values,how="left",left_index=True,right_index=True)
		data.append(values)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=["sample","start_codon","stop_codon"]
		temp.to_csv(outFile,sep="\t",index=0)
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_codon","stop_codon"])
		temp.to_csv(outFile,sep="\t",index=0)

def write_hydropathy_or_charge__per_codon(inFastaAttr,outFile):
	for fasta in inFastaAttr:
		with open(outFile+"_"+fasta.fastaLegend+"_values_at_each_position.txt",'w') as f:
			f.write("%s\t%s\n" %("transcripts","values"))
			for trans,perCodon in fasta.valuePerCodon.items():
				f.write("%s\t" %(trans))
				for pos in range(len(perCodon)):
					f.write("%s\t" %(str(perCodon[pos])))
				f.write("\n")

def main():
	"""main program"""
	parsed=create_parser_for_hydropathy_or_charge()
	(options,args)=parsed.parse_args()
	(transcriptFiles, output_prefix, upstream_codon, downstream_codon,index,trans_file_legend,table) = (options.transcriptFiles.strip().split(','),
	options.output_prefix,options.upstream_codon,options.downstream_codon,options.index,options.trans_file_legend.strip().split(','),options.geneticCode)
	if not index:
		raise IOError("Please input the hydropathy or charge index file")
	print("your input : "+ str(len(transcriptFiles))+" transcript files",file=sys.stderr)
	## handle bam file attr
	fasta_attr=[]
	for ii,jj in zip(transcriptFiles,trans_file_legend):
		fasta=fasta_attrbution(ii,jj)
		fasta_attr.append(fasta)
	data=pd.read_csv(index,sep="\t")
	index_dict={i:j for i,j in zip(data.iloc[:,0],data.iloc[:,2])}
	print("Start calculation...",file=sys.stderr)
	for fasta in fasta_attr:
		(fasta.startDensity,fasta.stopDensity,fasta.valuePerCodon) = hydropathy_or_charge(index_dict,fasta.fastaName,upstream_codon,downstream_codon,table)
	write_hydropathy_or_charge_dataframe(fasta_attr,output_prefix+"_values_dataframe.txt")
	write_hydropathy_or_charge__per_codon(fasta_attr,output_prefix)
	print("Finish the calculation!",file=sys.stderr)




if __name__ == "__main__":
		main()
















#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
the input are just the same as MetageneAnalysis.py but with longest.transcripts.cds.sequence.fa file input also which could be generated by GetProteinCodingSequence.py
output: pausing score files like this:
motif   si_3e_1_80S
MAA     0.0
AAT     0.0
ATA     0.0
TAL     0.0
ALL     0.0
LLE     0.0
LEA     0.0

each file has two columns:
col0: motifs
col1: pausing score.
'''

from .FunctionDefinition import *
from itertools import groupby
from collections import defaultdict
from functools import reduce
import re



def pausing_score(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,inCDS_countsFilterParma,inCDS_lengthFilterParma,transcriptFasta,left_pos,right_pos,mode,bamLegend,table,output_prefix):
	'''calculate the pausing score of all tri-peptide motif'''
	if left_pos and (not right_pos):
		raise IOError("Please input the right position match with the left position!")
	if (not left_pos) and right_pos:
		raise IOError("Please input the left position match with the right position!")
	transcript_sequence=fastaIter(transcriptFasta)
	pysamFile=pysam.AlignmentFile(in_bamFile,'rb')
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	in_selectTrans=in_selectTrans.intersection(transcript_sequence.keys()).intersection(in_startCodonCoorDict.keys())
	fout=open(output_prefix+"_"+bamLegend+"_pausing_score.txt",'w+')
	fout.write("%s\t%s" %("motif",bamLegend))
	transcript_used=0
	all_counts=0
	motif_num=0
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads

	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
		rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
		CDS_seq=transcript_sequence[trans][:-3]
		AA_seq=translation(CDS_seq,table=table,cds=False)
		(read_counts,read_counts_frameSum,trans_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		read_counts_frameSum_normed=10**6*(read_counts_frameSum/all_counts)
		sumValue=np.sum(read_counts_frameSum_normed)
		if left_pos and right_pos:
			if right_pos <= left_pos:
				raise IOError("The right position must be larger than the left position!")
			AA_seq_to_use=AA_seq[int(left_pos-1):right_pos] ## 25-75-----> [24:75]
			read_counts_frame=np.array(read_counts_frameSum_normed[int(left_pos-1):right_pos])
		else:
			AA_seq_to_use=AA_seq
			read_counts_frame=np.array(read_counts_frameSum_normed)
		if len(read_counts_frame) != len(AA_seq_to_use):
			# print("the lenght of density and AA are: "+ str(len(read_counts_frame))+"\t" + str(len(AA_seq_to_use)))
			continue
			# raise EOFError('The lenght of AA_seq is not equal to the lenght of read counts density')
		for i in np.arange(0,len(read_counts_frame),1):
			tmpScore=np.sum(read_counts_frame[i:i+3])/sumValue
			tmpMotif=''.join(AA_seq_to_use[i:i+3])
			fout.write("\n%s\t%s" %(tmpMotif,tmpScore))
			motif_num+=1
		transcript_used+=1
	fout.close()
	pysamFile.close()
	print("Start processing the sample " + str(in_bamFile),file=sys.stderr)
	print("The number of transcripts used for pausing score calulation is :" +str(transcript_used),file=sys.stderr)
	print("The number of motifs used for pausing score calculation is :" +str(motif_num),file=sys.stderr)


def filter_transcripts(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,inCDS_countsFilterParma,inCDS_lengthFilterParma,transcriptFasta,left_pos,right_pos,mode,table):
	'''
	filters:

	'''
	transcript_sequence=fastaIter(transcriptFasta)
	pysamFile=pysam.AlignmentFile(in_bamFile,'rb')
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	in_selectTrans=in_selectTrans.intersection(transcript_sequence.keys()).intersection(in_startCodonCoorDict.keys())
	filter_1=0
	filter_2=0
	filter_3=0
	filter_4=0
	filter_5=0
	passTransSet=set()
	all_counts=0
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads

	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
		rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
		CDS_seq=transcript_sequence[trans][:-3]## do not contain the stop codon
		AA_seq=translation(CDS_seq,table=table,cds=False)
		if len(CDS_seq) % 3 !=0:
			filter_1+=1
			continue
		if len(CDS_seq) < inCDS_lengthFilterParma:
			filter_2+=1
			continue
		(read_counts,read_counts_frameSum,trans_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		cds_reads_normed=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)*3))
		if mode == 'RPKM':
			if cds_reads_normed < inCDS_countsFilterParma:
				filter_3+=1
				continue
		if mode == 'counts':
			if cds_reads < inCDS_countsFilterParma:
				filter_3+=1
				continue

		sumValue=np.sum(read_counts_frameSum)
		sumValue_normed=10**9*(sumValue/(all_counts*len(read_counts_frameSum)*3))
		if sumValue_normed==0:
			filter_4+=1
			continue
		if len(read_counts_frameSum) != len(AA_seq):
			filter_5+=1
			# print("the lenght of density and AA are: "+ str(len(read_counts_frameSum))+"\t" + str(len(AA_seq)))
			continue
			# raise EOFError('The lenght of AA_seq is not equal to the lenght of read counts density')
		passTransSet.add(trans)
	pysamFile.close()
	print("The number of transcripts filtered by filter1 (len(CDS_seq)%3!=0) is : " + str(filter_1),file=sys.stderr)
	print("The number of transcripts filtered by filter2 (len(CDS_seq)<inCDS_lengthFilterParma) is :" + str(filter_2),file=sys.stderr)
	print("The number of transcripts filtered by filter3 (cds_reads < inCDS_countsFilterParma(RPKM)) is : "+str(filter_3),file=sys.stderr)
	print("The number of transcripts filtered by filter4 (sum(cds_reads)==0) is : " + str(filter_4),file=sys.stderr)
	print("The number of transcripts filtered by filter5 (annotated imcomplete) is : " + str(filter_5),file=sys.stderr)
	print("The number of transcripts used for pausing score calulation in " + str(in_bamFile) + "is: "+str(len(passTransSet)),file=sys.stderr)
	return passTransSet


def parse_args_for_pausing_score_calculation():
	parsed=create_parse_for_pausing_score_calculation()
	(options,args)=parsed.parse_args()
	(output_prefix, min_cds_codon,min_cds_counts,left_position, right_position, transcript_fasta,mode,table) = (
	options.output_prefix,options.min_cds_codon,options.min_cds_counts,options.left_position,options.right_position,
	options.transcript_fasta,options.mode,options.geneticCode)

	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	## get the common genes among all samples
	select_trans_list=[]
	for bamfs in bam_attr:
		passTransSet=filter_transcripts(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,
		stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,min_cds_counts,min_cds_codon,transcript_fasta,left_position,right_position,mode,table)
		select_trans_list.append(passTransSet)
	select_trans=reduce(set.intersection,select_trans_list) ## common transcripts among all samples which are all filtered.
	select_trans_out=pd.DataFrame(list(select_trans),columns=['transcript_id'])
	select_trans_out.to_csv(output_prefix+"_passed_transcripts.txt",index=0)
	print("Finish the transcripts filtering...",file=sys.stderr)
	print("The number of transcripts used for final pausing score calculaiton is : " + str(len(select_trans)),file=sys.stderr)
	for bamfs in bam_attr:
		pausing_score(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,
		stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,min_cds_counts,min_cds_codon,transcript_fasta,left_position,right_position,mode,bamfs.bamLegend,table,output_prefix)
	print("Finish the step of pausing score calculation",file=sys.stderr)


def main():
	parse_args_for_pausing_score_calculation()

if __name__ == "__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
Note:
1) -g: group names of your samples separated by comma. e.g. si-Ctrl,si-eIF3e. And the first one best to be the control group.
2) -r: replicate names of each group. groups separated by "__", and replicates separated by comma. e.g. si_ctrl_1_80S,si_ctrl_2_80S,si_ctrl_3_80S__si_3e_1_80S,si_3e_2_80S,si_3e_3_80S
'''


import pandas as pd
import numpy as np
import matplotlib
import sys
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from matplotlib.backends.backend_pdf import PdfPages
import seaborn as sns
from collections import defaultdict
from optparse import OptionParser
from .__init__ import __version__

def create_parser_for_density_on_each_kind_of_AA_codon():
	'''argument parser.'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action='store',type='string',dest='density_file',
		help='Input file in dataframe format (pd.DataFrame). Generated by RiboDensityAtEachKindAAOrCodon.py')
	parser.add_option('-o','--output',action='store',type='string',dest='output_prefix',
		help='Output files')
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",
		help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",
		help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,
		help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,
		help="The min of ylim. default=%default")
	parser.add_option("--level",action="store",type='string',dest='level',default="codon",
		help="Control the plot in codon level or AA level. default=%default")
	return parser

def calculate_mean_density(data,groups,replicates,output_prefix):
	'''calculate the mean values among different replicates. the input data have N+2 columns, N is the number of samples
	1) col 1: AAs
	2) col2: codons
	3) col3-colN: different samples
	4) suppose we just have two groups (if you have more than two groups, split it up and analysis every two groups.[one is control and the other is treated])
	'''
	label_dict={}
	data_dict={}
	data_mean_dict=defaultdict(dict)
	AAs=data.iloc[:,0]
	codons=data.iloc[:,1]
	for g,r in zip(groups,replicates):
		label_dict[g]=r.strip().split(',')
	## separate the data into different groups
	for g in groups:
		data_dict[g]=data.loc[:,label_dict[g]]

	for g in groups:
		for i in np.arange(data.shape[0]):
			density=np.mean(data_dict[g].loc[i,label_dict[g]])
			data_mean_dict[g][i]=density
	## transform the dict to a python dataframe
	for g in groups:
		data_mean_dict[g]=pd.DataFrame(data_mean_dict[g],index=[g]).T
	## concatenate different data frame
	data_mean=pd.concat([v for v in data_mean_dict.values()],axis=1)
	data_mean=pd.concat((AAs,codons,data_mean),axis=1)
	## write the mean density file
	data_mean.to_csv(output_prefix+"_mean_codon_density_dataframe.txt",sep="\t",index=0)
	return data_mean

def reshape_mean_data(data_mean,groups):
	''' this requires groups only have two conditions: the first one is control group and the second one is the treat group'''
	Non_polar_AA=['A','P','G','V','W','I','F','M','L']
	Polar_AA=['T','Y','Q','C','N','S']
	Positive_AA=['K','H','R']
	Negative_AA=['D','E']
	Non_polar_AA_color=['gray']*len(Non_polar_AA)
	Polar_AA_color=["purple"]*len(Polar_AA)
	Negative_AA_color=['blue']*len(Negative_AA)
	Positive_AA_color=['red']*len(Positive_AA)
	my_color=Non_polar_AA_color+Polar_AA_color+Negative_AA_color+Positive_AA_color
	data_mean_sorted=data_mean.sort_values('AA',inplace=False)
	ctrl_mean=data_mean_sorted[str(groups[0])].groupby(data_mean_sorted['AA']).sum()
	treat_mean=data_mean_sorted[str(groups[1])].groupby(data_mean_sorted['AA']).sum()
	data_mean_AA=pd.concat((ctrl_mean,treat_mean),axis=1)
	data_mean_AA['ratio']=(data_mean_AA.iloc[:,1]-data_mean_AA.iloc[:,0])/data_mean_AA.iloc[:,0]

	Non_polar_AA_data=data_mean_AA.loc[Non_polar_AA,:]
	Non_polar_AA_data=Non_polar_AA_data.sort_values(by="ratio",ascending=False)
	Polar_AA_data=data_mean_AA.loc[Polar_AA,:]
	Polar_AA_data=Polar_AA_data.sort_values(by="ratio",ascending=False)
	Negative_AA_data=data_mean_AA.loc[Negative_AA,:]
	Negative_AA_data=Negative_AA_data.sort_values(by="ratio",ascending=False)
	Positive_AA_data=data_mean_AA.loc[Positive_AA,:]
	Positive_AA_data=Positive_AA_data.sort_values(by="ratio",ascending=False)
	data_mean_AA_reshaped=pd.concat((Non_polar_AA_data,Polar_AA_data,Negative_AA_data,Positive_AA_data),axis=0)
	return data_mean_AA_reshaped,my_color

def plot_density_for_each_sample_codon_level(data,data_mean,groups,replicates,output_prefix,ymin,ymax):
	''' this requires groups only have two conditions: the first one is control group and the second one is the treat group'''
	plt.rc('font',weight='bold')
	data_sorted=data.sort_values('AA',inplace=False)
	data_mean_sorted=data_mean.sort_values('AA',inplace=False)
	data_mean_sorted['ratio']=(data_mean_sorted.iloc[:,3]-data_mean_sorted.iloc[:,2])/data_mean_sorted.iloc[:,2]
	codons_sort=data_sorted.iloc[:,1].values
	codons_mean_sort=data_mean_sorted.iloc[:,1].values
	replicates=','.join(replicates).split(',')
	with PdfPages(output_prefix + "_density_on_each_kind_of_codon.pdf") as pdf:
		for r in replicates:
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			sns.barplot(x=np.arange(len(codons_sort)),y=data_sorted.loc[:,r],palette='husl',ax=ax)
			ax.set_xlabel("codons",fontdict={'size':20,'weight':"bold","family":"Arial"})
			ax.set_ylabel("Relative ribosome density",fontdict={'size':20,'weight':"bold","family":"Arial"})
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['bottom'].set_linewidth(2)
			ax.spines['left'].set_linewidth(2)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			else:
				pass
			ax.tick_params(which="both",width=2)
			plt.xticks(np.arange(len(codons_sort)),codons_sort,rotation=270,fontsize=12,fontweight="bold")
			for a,b,c in zip(np.arange(len(codons_sort)),data_sorted.loc[:,r],data_sorted['AA']):
				if b>0:
					plt.text(a,b,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
				else:
					plt.text(a,float(b)-0.012,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
			plt.title(r,fontdict={'size':20,'weight':"bold","family":"Arial"})
			# plt.tight_layout()
			pdf.savefig(fig)
			plt.close()
		for r in data_mean_sorted.columns[2:]:
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			sns.barplot(x=np.arange(len(codons_mean_sort)),y=data_mean_sorted.loc[:,r],palette='husl',ax=ax)
			ax.set_xlabel("codons",fontdict={'size':20,'weight':"bold","family":"Arial"})
			min_ylim=min(plt.ylim())
			if r == 'ratio':
				ax.axhline(0,color='k',clip_on=False,linewidth=1)
				ax.set_ylabel("Difference of ribosome density.\n"+ "("+str(groups[1])+"-"+str(groups[0])+")"+"/"+str(groups[0]),fontdict={'size':20,'weight':"bold","family":"Arial"})
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
				for a,b,c in zip(np.arange(len(codons_mean_sort)),data_mean_sorted.loc[:,r],data_mean_sorted['AA']):
					if b>0:
						plt.text(a,b,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
					else:
						plt.text(a,float(b)-0.012,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
					plt.vlines(a,ymin=min_ylim,ymax=0,color='gray',linestyles='dashed',alpha=0.5)
			else:
				ax.set_ylabel("Relative ribosome density",fontdict={'size':20,'weight':"bold","family":"Arial"})
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				else:
					pass
				for a,b,c in zip(np.arange(len(codons_mean_sort)),data_mean_sorted.loc[:,r],data_mean_sorted['AA']):
					if b>0:
						plt.text(a,b,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
					else:
						plt.text(a,float(b)-0.012,c,ha='center',va='bottom',fontdict={'size':10,'weight':"bold","family":"Arial"})
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['bottom'].set_linewidth(2)
			ax.spines['left'].set_linewidth(2)
			ax.tick_params(which="both",width=2)
			plt.xticks(np.arange(len(codons_mean_sort)),codons_mean_sort,rotation=270,fontsize=12,fontweight="bold")
			plt.title(r,fontdict={'size':20,'weight':"bold","family":"Arial"})
			pdf.savefig(fig)
			plt.close()

def plot_density_for_each_sample_AA_level(data,data_mean_reshaped,groups,replicates,colors,output_prefix,ymin,ymax):
	plt.rc('font',weight='bold')
	data_sorted=data.sort_values('AA',inplace=False)
	data_sorted=data_sorted.groupby(data_sorted['AA']).sum()
	replicates=','.join(replicates).split(',')
	AAs=data_sorted.index.values
	AAs_index=data_mean_reshaped.index
	with PdfPages(output_prefix + "_density_on_each_kind_of_AA.pdf") as pdf:
		for r in replicates:
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			sns.barplot(x=np.arange(len(AAs)),y=data_sorted.loc[:,r],palette='husl',ax=ax,alpha=0.9)
			ax.set_xlabel("AAs",fontdict={'size':20,'weight':"bold","family":"Arial"})
			ax.set_ylabel("Relative ribosome density",fontdict={'size':20,'weight':"bold","family":"Arial"})
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['bottom'].set_linewidth(2)
			ax.spines['left'].set_linewidth(2)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			else:
				pass
			ax.tick_params(which="both",width=2)
			plt.xticks(np.arange(len(AAs)),AAs,fontsize=12,fontweight='bold')
			plt.title(r,fontdict={'size':20,'weight':"bold","family":"Arial"})
			# plt.tight_layout()
			pdf.savefig(fig)
			plt.close()
		for r in data_mean_reshaped.columns:
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			labels_legend=['non polar','polar','positively charged','negatively charged']
			colors_legend=['gray','purple','red','blue']
			patches = [ mpatches.Patch(color=colors_legend[i], label=labels_legend[i] ) for i in range(len(colors_legend)) ]
			sns.barplot(x=np.arange(len(AAs_index)),y=data_mean_reshaped.loc[:,r],palette=colors,ax=ax,alpha=0.9)
			ax.set_xlabel("AAs",fontdict={'size':20,'weight':"bold","family":"Arial"})
			if r == 'ratio':
				ax.axhline(0,color='k',clip_on=False,linewidth=1)
				ax.set_ylabel("Difference of ribosome density.\n"+ "("+str(groups[1])+"-"+str(groups[0])+")"+"/"+str(groups[0]),fontdict={'size':20,'weight':"bold","family":"Arial"})
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
			else:
				ax.set_ylabel("Relative ribosome density",fontdict={'size':20,'weight':"bold","family":"Arial"})
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				else:
					pass
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['bottom'].set_linewidth(2)
			ax.spines['left'].set_linewidth(2)
			ax.tick_params(which="both",width=2)
			plt.xticks(np.arange(len(AAs_index)),AAs_index,fontsize=12,fontweight="bold")
			plt.title(r,fontdict={'size':20,'weight':"bold","family":"Arial"})
			plt.legend(handles=patches,loc='best',prop={'size':20,'weight':"bold","family":"Arial"})
			# plt.tight_layout()
			pdf.savefig(fig)
			plt.close()


def main():
	parser=create_parser_for_density_on_each_kind_of_AA_codon()
	(options,args)=parser.parse_args()
	(data,output_prefix,groups,replicates,level,ymin,ymax)=(options.density_file,options.output_prefix,options.group_name.strip().split(','),
	options.replicate_name.strip().split('__'),options.level,options.ymin,options.ymax)
	##  input data
	data=pd.read_csv(data,sep='\t')
	data_mean=calculate_mean_density(data,groups,replicates,output_prefix)
	data_mean_reshaped,colors_for_AA=reshape_mean_data(data_mean,groups)
	if level == 'codon':
		print("Start the step of plotting...",file=sys.stderr)
		plot_density_for_each_sample_codon_level(data,data_mean,groups,replicates,output_prefix,ymin,ymax)
		print('finishing the plot step!',file=sys.stderr)
	elif level == 'AA':
		print("Start the step of plotting...",file=sys.stderr)
		plot_density_for_each_sample_AA_level(data,data_mean_reshaped,groups,replicates,colors_for_AA,output_prefix,ymin,ymax)
		print('finishing the plot step!',file=sys.stderr)
	else:
		raise IOError("Please reset your --level parameter.[AA or codon]")


if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
'''

import sys
import pysam
from itertools import groupby
import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from optparse import OptionParser
from collections import defaultdict



def create_parser_for_reads_length():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage)
	parser.add_option("-i","--input",action="store",type="string",dest="bamFile",help="Input file in BAM format")
	parser.add_option("-l","--left-position",action="store",type="int",default=None, dest="left_position", help="Left position of the sequence you are interested in.")
	parser.add_option("-r","--right-position",action="store",type="int",default=None,dest="right_position",help="Right position of the sequence you are interested in.")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',default=None,
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name. default=%default")
	parser.add_option("--type",action='store',type='string',dest='type',default='CDS',help='Type of regions.[CDS/5UTR/3UTR].default=%default')
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	return parser



def reload_transcripts_information(longestTransFile):
	selectTrans=set()
	transLengthDict={}
	cdsLengthDict={}
	startCodonCoorDict={}
	stopCodonCoorDict={}
	transID2geneID={}
	transID2geneName={}
	with open(longestTransFile,'r') as f:
		for line in f:
			if line.strip()=='':
				continue
			if line.strip().split("\t")[0] == 'trans_id':
				continue
			transID=line.strip().split("\t")[0]
			geneID=line.strip().split("\t")[2]
			geneName=line.strip().split("\t")[3]
			startCodon=int(line.strip().split("\t")[7])
			stopCodon=int(line.strip().split("\t")[8])
			cds_length=int(line.strip().split("\t")[9])
			transLength=int(line.strip().split("\t")[12])
			selectTrans.add(transID)
			transLengthDict[transID]=transLength
			startCodonCoorDict[transID]=startCodon
			stopCodonCoorDict[transID]=stopCodon
			transID2geneID[transID]=geneID
			transID2geneName[transID]=geneName
			cdsLengthDict[transID]=cds_length
			# print(transID,geneID,geneName,startCodon,stopCodon,transLength)
	print(str(len(selectTrans))+'  transcripts will be used in the follow analysis.\n', file=sys.stderr)
	return selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict

def IDTransform(coorFile,in_selectTrans,id_type):
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if in_selectTrans:
		select_trans=pd.read_csv(in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif id_type == 'gene_name' or id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	return select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,cdsLengthDict

def OutputReadsLength(in_bamFile,in_selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,left_position,right_position,Type):
	readLengthList=[]
	pysamFile=pysam.AlignmentFile(in_bamFile,'rb')
	if not in_selectTrans:
		selectTrans=pysamFile.references
	elif in_selectTrans:
		selectTrans=set(pysamFile.references).intersection(in_selectTrans)
	for trans in selectTrans:
		leftCoor =int(startCodonCoorDict[trans])-1
		rightCoor=int(stopCodonCoorDict[trans])-3
		for record in pysamFile.fetch(trans):
			if record.flag==16 or record.flag==272:
				continue
			pos=record.pos
			readLength=record.query_length
			if Type.upper()=='5UTR':
				if pos < leftCoor:
					readLengthList.append(readLength)
			elif Type.upper()=='3UTR':
				if pos > rightCoor:
					readLengthList.append(readLength)
			elif Type.upper()=='CDS':
				if left_position and right_position:
					if pos>=(leftCoor+left_position-1) and pos <= (leftCoor+right_position):
						readLengthList.append(readLength)
				elif not left_position and not right_position:
					if pos >=leftCoor and pos < rightCoor:
						readLengthList.append(readLength)
				else:
					raise IOError("Please input -l and -r parameters!")
			else:
				raise IOError("--type [CDS|5UTR|3UTR]")
	return readLengthList

def plot_reads_length(readLengthList,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	'''
	plot length distribution
	'''
	values=readLengthList
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	plt.hist(values,bins=40,color="b",width=0.5,alpha=0.9)
	ax.spines["top"].set_linewidth(2)
	ax.spines["right"].set_linewidth(2)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.set_xlabel("Length of reads",fontdict=text_font)
	ax.set_ylabel("Read Counts",fontdict=text_font)
	# ax.set_xticks(np.arange(0,50,5))
	# ax.set_xticklabels((np.arange(0,50,5)))
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.tight_layout()
	plt.savefig(output_prefix+"_reads_length.pdf")
	plt.close()

def parse_reads_length():
    parser=create_parser_for_reads_length()
    (options,args)=parser.parse_args()
    (bamFile,left_position,right_position,output_prefix,coorFile,in_selectTrans,Type,id_type)=(options.bamFile,options.left_position,options.right_position,options.output_prefix,options.coorFile,options.in_selectTrans,options.type,options.id_type)
    select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,cdsLengthDict=IDTransform(coorFile,in_selectTrans,id_type)
    print("Starting...")
    readLengthList=OutputReadsLength(bamFile,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,left_position,right_position,Type)
    plot_reads_length(readLengthList,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"})
    readLengthListDB=pd.DataFrame(readLengthList)
    readLengthListDB.to_csv(output_prefix+"_"+Type+"_read_length.txt",index=0,header=0)
    print("Finish!")

def main():
	parse_reads_length()

if __name__=="__main__":
    main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
'''


from .FunctionDefinition import *
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib import gridspec
import matplotlib.colors as cols
import seaborn as sns


def calculate_polarity(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,min_cds_counts):
	""" calculate the polarity of every gene (the longest transcript as for the final gene)
		1) calculate the density of ribosome at all positions of a gene , d
		2) calculate the normalized distance from the center of a gene at position i, w
		3) calculate the polarity at all positions of a gene ,pi and sum up the pi as the final polarity of the gene

		pi = di*wi/sum(di)
		wi = (2*i-(len(gene)+1))/(len(gene)-1)
	"""
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	GeneNum = len(in_selectTrans)
	polarityScore=[]
	polarityScorePerTrans=[]
	passTransSet={}
	filter=0
	for trans in in_selectTrans:
		GeneNum-+1
		leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
		rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
		(read_counts,read_counts_frameSum,trans_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		## get read density in nt unit
		read_counts_frame=read_counts[(leftCoor+15):(rightCoor-15)] ##  exclude the first 5 and last f codons
		tmpDistanceNormed=np.arange(1,len(read_counts_frame)+1)
		DistanceNormed=(2*tmpDistanceNormed-(len(read_counts_frame)+1))/(len(read_counts_frame)-1)
		sumValue=np.sum(read_counts_frame)
		if sumValue < min_cds_counts :
			filter+=1
			continue
		if sumValue == 0:
			tmpPolarityScorePerTrans=read_counts_frame
		else:
			tmpPolarityScorePerTrans=np.multiply(read_counts_frame,DistanceNormed)/np.sum(read_counts_frame)
		polarityScorePerTrans.append(tmpPolarityScorePerTrans)
		passTransSet[trans]=np.sum(tmpPolarityScorePerTrans)
	## calculate polarity of each gene
	for i in range(len(polarityScorePerTrans)):
		polarityScore.append(np.sum(polarityScorePerTrans[i]))
	pysamFile.close()
	print("Metaplots Transcript Number for bam file"+in_bamFile+" is :"+str(len(passTransSet)),file=sys.stderr)
	print("The number of genes filtered by read counts equals to "+ str(min_cds_counts)+" is: " + str(filter),file=sys.stderr)
	print("There are "+str(len(polarityScore))+" genes used for plot polarity value",file=sys.stderr)
	return polarityScore,passTransSet

def DrawPolarity(data,inOutPrefix):
	"""plot polarity scores"""
	text_font={"size":20,"family":"Arial","weight":"bold"}
	legend_font={"size":10,"family":"Arial","weight":"bold"}
	samples=np.unique(data.columns)
	plt.rc('font',weight='bold')
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	## try to use for loop to re-write the plot function
	for i in np.arange(len(samples)):
		lst=data.iloc[:,i].values
		lst=lst[~np.isnan(lst)]
		sns.distplot(lst,hist=False,rug=False,label=samples[i],color=colors[i])
	ax.set_xlabel("Polarity score",fontdict=text_font)
	ax.set_ylabel("Relative gene numbers",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,length=2)
	plt.legend(loc="best",prop=legend_font)
	plt.tight_layout()
	plt.savefig(inOutPrefix+"_polarity.pdf")
	plt.close()

def write_bam_file_polarity_dataframe(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		d=bms.passTransSet
		i=bms.bamLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")
	return data

def parse_args_for_polarity_calculation():
	parsed=create_parser_for_polarity_calculation()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	print("There are "+str(len(select_trans))+" transcripts with both start and stop codon will be used for following analysis.",file=sys.stderr)

	for bamfs in bam_attr:
		(bamfs.polarity,bamfs.passTransSet)=calculate_polarity(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,options.min_cds_counts)
	print("Finish the step of calculate the polarity of genes",file=sys.stderr)
	data_polarity=write_bam_file_polarity_dataframe(bam_attr,options.output_prefix+"_polarity_dataframe.txt")
	print("Finish the step of write_bam_file_polarity_dataframe",file=sys.stderr)
	if options.plot.upper() in ['YES','Y']:
		DrawPolarity(data_polarity,options.output_prefix)
		print("Finish the step of plotting!",file=sys.stderr)
	elif options.plot.upper() in ['NO','N','NONE']:
		pass
	else:
		raise IOError("Please input a correct --plot parameter! [yes/no]")

def main():
	"""main program"""
	parse_args_for_polarity_calculation()

if __name__ == "__main__":
		main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
Usage: ModifyHTseq -i bamFile -g gtfFile -o countsFile -t exon -m union -q 10 --minLen 25 --maxLen 35 --exclude-first 45 --exclude-last 15 --id-type gene_id
Notes: This script only used for strand specific library.
'''


from __future__ import division
import numpy
import sys
import pysam
import os
import itertools
import HTSeq
from optparse import OptionParser
from .__init__ import __version__
# __version__="0.1"


def create_parse_for_htseq():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bamFile",
			help="Input file in bam format. default=%default")
	parser.add_option("-g","--gtfFile",action="store",type="string",dest="gtfFile",
			help="Standard GTF file of a specific species.")
	parser.add_option("-o","--outputFile",action="store",type="string",dest="outputFile",
			help="File name of output files.")
	parser.add_option("-t","--type",action="store",type="string",dest="Type",default='exon',
			help="Feature type (3rd column in GFF file) to be used. [exon or CDS]")
	parser.add_option("-m","--mode",action="store",type="string",default='union',dest="mode",
			help="mode to handle reads overlapping more than one feature, the same as htseq-count [union,intersection-strict,intersection-nonempty]. default=%default")
	parser.add_option("-q","--min-quality",action="store",type="int",dest="minQuality",default=10,
			help="The minimum quality of base to be required! default=%default")
	parser.add_option('--minLen',action="store",type="int",dest="minLen",default=25,
			help="The minimum length of reads to be considered. default=%default(nt)")
	parser.add_option('--maxLen',action="store",type="int",dest="maxLen",default=35,
			help="The max length of reads to be considered. default=%default(nt)")
	parser.add_option('--exclude-first',action="store",type="int",dest="excludeFirst",default=45,
			help="The number of nucleotides need to be excluded from start codon. default=%default(nt)=15(codon)")
	parser.add_option('--exclude-last',action="store",type="int",dest="excludeLast",default=15,
			help="The number of nucleotides need to be excluded from stop codon. default=%default(nt)=5(codon)")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="gene_id",
			help="define the id type users input. the default is gene id, the same as '-i' in htseq-count. default=%default")
	return parser

def modifHTSeq(bam_filename, gff_filename, out_file, overlap_mode, feature_type, id_attribute, minaqual, exclude_start_distance, exclude_stop_distance, min_len, max_len):
	#feature GenomicArrayOfSets
	features = HTSeq.GenomicArrayOfSets( "auto", stranded=True)
	counts = {}
	start_codon_sites = {}
	stop_codon_sites = {}
	#GTF
	gff = HTSeq.GFF_Reader( gff_filename, end_included=True)
	i = 0
	for f in gff:
		if f.type == feature_type:
			if id_attribute in f.attr : #the same to the f.attr.keys()
				feature_id = f.attr[ id_attribute ] # f.attr will return the 9-th colum of the input gtf file as {}
			else:
				feature_id = f.attr[ 'gene_id' ] #in the gtf file of Rat, there are some CDS/exon dont have gene_name ,but every items have gene_id
			features[ f.iv ] += feature_id #label the chrmosome with gene_name, if dont have gene_name,replaced by gene_id
			#counts[ f.attr[ id_attribute ] ] = 0 #only counts reads for genes with id_attribute, so cant repaced by counts[ feature_id ] = 0
			counts[ feature_id ] = 0
		### if there are multiple TIS, use the most 5' end start codon and the most 3' end stop codon
		if f.type == "start_codon":
			if id_attribute in f.attr :
				gname=f.attr[ id_attribute ]
			if gname not in start_codon_sites :
				start_codon_sites[gname] = f.iv.start_d
			else :
				if f.iv.strand == "+":
					start_codon_sites[gname] = min(f.iv.start_d, start_codon_sites[gname])
				else :
					start_codon_sites[gname] = max(f.iv.start_d, start_codon_sites[gname])
		#
		if f.type == "stop_codon":
			if id_attribute in f.attr :
				gname=f.attr[ id_attribute ]
			if gname not in stop_codon_sites :
				stop_codon_sites[gname] = f.iv.end_d
			else :
				if f.iv.strand == "+":
					stop_codon_sites[gname] = max(f.iv.end_d, stop_codon_sites[gname])
				else :
					stop_codon_sites[gname] = min(f.iv.end_d, stop_codon_sites[gname])
		i += 1
		if i % 100000 == 0 :
			sys.stderr.write( "%d GFF lines processed.\n" % i )
	#bam
	read_seq = HTSeq.BAM_Reader( bam_filename )
	#counts
	empty = 0
	ambiguous = 0
	notaligned = 0
	lowqual = 0
	nonunique = 0
	i = 0
	for r in read_seq:
		if i > 0 and i % 100000 == 0 :
			sys.stderr.write( "%d SAM alignment record processed.\n" % i )
		i += 1
		if not r.aligned:
			notaligned += 1
			continue
		if r.optional_field( "NH" ) > 1:
			nonunique += 1
			continue
		if r.aQual < minaqual:
			lowqual += 1
			continue
		###
		if len(r.read.seq) < min_len or len(r.read.seq) > max_len:
			continue
		iv_seq = ( co.ref_iv for co in r.cigar if co.type == "M" and co.size > 0 )
		if overlap_mode == "union":
			fs = set()
			for iv in iv_seq:
				for iv2, fs2 in features[ iv ].steps():
					fs = fs.union( fs2 )
		elif overlap_mode == "intersection-strict" or overlap_mode == "intersection-nonempty":
			fs = None
			for iv in iv_seq:
				for iv2, fs2 in features[ iv ].steps():
					if len(fs2) > 0 or overlap_mode == "intersection-strict":
						if fs is None:
							fs = fs2.copy()
						else:
							fs = fs.intersection( fs2 )
		else:
			sys.exit( "Illegal overlap mode." )
		if fs is None or len( fs ) == 0:
			empty += 1
		elif len( fs ) > 1:
			ambiguous += 1
		else:
			try : #some genes may dont have start or stop codon
				if abs(start_codon_sites[ list(fs)[0] ] - r.iv.start_d) < exclude_start_distance :
					continue
				elif abs(r.iv.end_d - stop_codon_sites[ list(fs)[0] ]) < exclude_stop_distance:
					continue
				else :
					counts[ list(fs)[0] ] += 1
			except :
				counts[ list(fs)[0] ] += 1
	#output
	with open(out_file,"w") as fout:
		fout.write("%s\t%s\n" %(id_attribute.strip(),"count"))
		for fn in sorted( counts.keys() ):
			fout.write("%s\t%s\n" % (fn,counts[fn]))
		fout.write("__no_feature\t%d\n" % empty)
		fout.write("__ambiguous\t%d\n" % ambiguous)
		fout.write("__too_low_aQual\t%d\n" % lowqual)
		fout.write("__not_aligned\t%d\n" % notaligned)
		fout.write("__alignment_not_unique\t%d\n" % nonunique)


def main():
    parser=create_parse_for_htseq()
    (options,args)=parser.parse_args()
    (bamFile,gtfFile,outFile,modeType,featureType,idAttribute,minQual,excludeStartDistance,excludeStopDistance,minLen,maxLen)=(options.bamFile,options.gtfFile,options.outputFile,
    options.mode,options.Type,options.id_type,options.minQuality,options.excludeFirst,options.excludeLast,options.minLen,options.maxLen)
    print("Start read counting...",file=sys.stderr)
    modifHTSeq(bamFile,gtfFile,outFile,modeType,featureType,idAttribute,minQual,excludeStartDistance,excludeStopDistance,minLen,maxLen)
    print("Finish read counting!",file=sys.stderr)

if __name__=="__main__":
    main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
usage: python MergeSampleDensity.py inputFile outputFile
1) inputFile: input files separated by comma. e.g. test1_dataframe.txt,test2_dataframe.txt
2) outputFile: an output file after merging.
'''
import os
import sys
import numpy as np
import pandas as pd

inputFile=sys.argv[1]
outputFile=sys.argv[2]

def MergeSampleData(inputFile,outputFile):
    inputFiles=inputFile.strip().split(',')
    data=[pd.read_csv(File,sep="\t",header=0) for File in inputFiles]
    Final_data=pd.concat(data,axis=0)
    Final_data.to_csv(outputFile,sep="\t",index=0)

def main():
    print("Start merging...",file=sys.stderr)
    MergeSampleData(inputFile,outputFile)
    print("Finish the files merging!",file=sys.stderr)
if __name__=="__main__":
    main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Description: This script is used for hydropathy or charge plot
'''

import numpy as np
import pandas as pd
import sys
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from functools import reduce
from .__init__ import __version__

def create_parser_for_hydropathy_plot():
	'''argument parser'''
	usage="usage: python %prog [options]" +"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="density_file",help="Input file in txt format.And the files has three columns; column 1: sample;columns 2: start_density; column 3: stop_density")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon", help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option("-l",'--left-position',action="store",type="int",dest='left_position',default=None,help="The left position of the region that you are interested in. default=%default")
	parser.add_option("-r",'--right-position',action='store',type='int',dest='right_position',default=None,help="The right position of the region that you are interested in. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="Control the mode for plot.[all or single]. default=%default")
	parser.add_option("--axvline",action="store",type="float",dest="axvline",default=None,help="Position to plot vetical line")
	parser.add_option("--ylab",action="store",type="string",dest="ylab",default="Average Index",help="labels for y axis.'Average hydrophobicity' or 'Average charges'")
	parser.add_option("--start",action="store",type="int",dest="start_position",default=5,help="The start position need to be averaged.default=%default")
	parser.add_option("--window",action="store",type="int",dest="window",default=7,help="The length of silde window. ddefault=%default")
	parser.add_option("--step",action="store",type='int',dest="step",default=1,help="The step length of slide window. default=%default")

	return parser

def plot_all_density(data,samples,type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,ymin,ymax,axvline,ylab=None,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution'''
	plt.rc("font",weight="bold")
	fig=plt.figure(figsize=(16,8))
	ax=fig.add_subplot(111)
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(samples))

	for i in np.arange(len(samples)):
		if type=="start codon":
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
			if axvline:
				ax.axvline(axvline,color='red',dashes=[3,2])
			else:
				pass

		else:
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))

	ax.set_xlabel("Distance from "+type + ' (codon)',fontdict=text_font)
	ax.set_ylabel(ylab,fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,labelsize=20)
	if not ymin and not ymax:
		pass
	elif not ymin and ymax:
		ax.set_ylim(0,ymax)
	elif ymin and not ymax:
		raise IOError("Please offer the ymax parameter as well!")
	elif ymin and ymax:
		ax.set_ylim(ymin,ymax)
	else:
		raise IOError("Please enter correct ymin and ymax parameters!")
	plt.legend(loc="best",prop=legend_font)
	# plt.legend(loc="best",prop={'size':10,'weight':'bold'})
	plt.tight_layout()
	plt.savefig(inOutPrefix+"_"+type+"."+inOutFomat,format=inOutFomat)
	plt.close()

def plot_density_for_each_sample(data,samples,type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,ymin,ymax,axvline,ylab=None,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the hydropathy'''
	plt.rc('font',weight='bold')
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	colors=["b","orangered","green","c","m","y","k","w"]
	for i in np.arange(len(samples)):
		if type=="start codon":
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
			ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
			ax.set_ylabel(ylab,fontdict=text_font)
			ax.spines["top"].set_visible(False)
			ax.spines["right"].set_visible(False)
			ax.spines["bottom"].set_linewidth(2)
			ax.spines["left"].set_linewidth(2)
			ax.tick_params(which="both",width=2,labelsize=20)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.legend(loc="best",prop=legend_font)
			plt.savefig(inOutPrefix+"_"+str(samples[i])+"_start_codon"+"."+inOutFomat,format=inOutFomat)
			plt.close()

		else:
			fig=plt.figure(figsize=(16,8))
			ax=fig.add_subplot(111)
			plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
			ax.set_xticks(np.arange(0,winLen,50))
			ax.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))
			ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
			ax.set_ylabel(ylab,fontdict=text_font)
			ax.spines["top"].set_visible(False)
			ax.spines["right"].set_visible(False)
			ax.spines["bottom"].set_linewidth(2)
			ax.spines["left"].set_linewidth(2)
			ax.tick_params(which="both",width=2,labelsize=20)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.legend(loc="best",prop=legend_font)
			plt.savefig(inOutPrefix+"_"+str(samples[i])+"_stop_codon"+"."+inOutFomat,format=inOutFomat)
			plt.close()
def plot_density_for_specific_region(data,samples,type,left_position,right_position,inOutPrefix,inOutFomat,ymin,ymax,axvline,mode,ylab=None,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution for a specific region'''
	plt.rc("font",weight="bold")
	winLen=right_position-left_position+1
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(samples))
	if mode == "all":
		fig=plt.figure(figsize=(16,8))
		ax=fig.add_subplot(111)
		for i in np.arange(len(samples)):
			if type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][(left_position-1):right_position],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen+5,5))
				ax.set_xticklabels(np.arange(left_position-1,right_position+5,5))
				if axvline:
					ax.axvline(axvline,color='red',dashes=[3,2])
				else:
					pass

			else:
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][(left_position-1):right_position],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen,5))
				ax.set_xticklabels((np.arange(0,winLen,5)-right_position))

		ax.set_xlabel("Distance from "+type + ' (codon)',fontdict=text_font)
		ax.set_ylabel(ylab,fontdict=text_font)
		ax.spines["top"].set_visible(False)
		ax.spines["right"].set_visible(False)
		ax.spines["bottom"].set_linewidth(2)
		ax.spines["left"].set_linewidth(2)
		ax.tick_params(which="both",width=2,labelsize=20)
		if not ymin and not ymax:
			pass
		elif not ymin and ymax:
			ax.set_ylim(0,ymax)
		elif ymin and not ymax:
			raise IOError("Please offer the ymax parameter as well!")
		elif ymin and ymax:
			ax.set_ylim(ymin,ymax)
		else:
			raise IOError("Please enter correct ymin and ymax parameters!")
		plt.legend(loc="best",prop=legend_font)
		# plt.legend(loc="best",prop={'size':10,'weight':'bold'})
		plt.tight_layout()
		plt.savefig(inOutPrefix+"_specific_region_"+type+"."+inOutFomat,format=inOutFomat)
		plt.close()
	elif mode == "single":
		for i in np.arange(len(samples)):
			if type=="start codon":
				fig=plt.figure(figsize=(16,8))
				ax=fig.add_subplot(111)
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][left_position-1:right_position],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen+5,5))
				ax.set_xticklabels(np.arange(left_position-1,right_position+5,5))
				if axvline:
					ax.axvline(axvline,color='red',dashes=[3,2])
				else:
					pass
				ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
				ax.set_ylabel(ylab,fontdict=text_font)
				ax.spines["top"].set_visible(False)
				ax.spines["right"].set_visible(False)
				ax.spines["bottom"].set_linewidth(2)
				ax.spines["left"].set_linewidth(2)
				ax.tick_params(which="both",width=2,labelsize=20)
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
				plt.legend(loc="best",prop=legend_font)
				plt.savefig(inOutPrefix+"_"+str(samples[i])+"_start_codon"+"."+inOutFomat,format=inOutFomat)
				plt.close()

			else:
				fig=plt.figure(figsize=(16,8))
				ax=fig.add_subplot(111)
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][left_position-1:right_position],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen+5,5))
				ax.set_xticklabels(np.arange(left_position-1,right_position+5,5))
				if axvline:
					ax.axvline(axvline,color='red',dashes=[3,2])
				else:
					pass
				ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
				ax.set_ylabel(ylab,fontdict=text_font)
				ax.spines["top"].set_visible(False)
				ax.spines["right"].set_visible(False)
				ax.spines["bottom"].set_linewidth(2)
				ax.spines["left"].set_linewidth(2)
				ax.tick_params(which="both",width=2,labelsize=20)
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
				plt.legend(loc="best",prop=legend_font)
				plt.savefig(inOutPrefix+"_"+str(samples[i])+"_stop_codon"+"."+inOutFomat,format=inOutFomat)
				plt.close()
	else:
		raise IOError("Please enter a correct --mode parameter![all or single]")

def slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,start,window,step):
	start_average=[]
	stop_average=[]
	label=[]
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	for i in np.arange(len(samples)):
			tmp1_data=np.zeros(winLen)
			tmp1_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][0:int(start)]
			tmp1_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][-int(start):]
			for j in np.arange(start,winLen-start,step):
				tmp1_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][(j-int((window-1)/2)):(j+int((window-1)/2))])
			start_average.extend(tmp1_data)
			label.extend([samples[i]]*winLen)

			tmp2_data=np.zeros(winLen)
			tmp2_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][0:int(start)]
			tmp2_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][-int(start):]
			for j in np.arange(start,winLen-start,step):
				tmp2_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][(j-int((window-1)/2)):(j+int((window-1)/2))])
			stop_average.extend(tmp2_data)

	data_average=pd.DataFrame([label,start_average,stop_average],index=['sample','start_density','stop_density'])
	data_average=data_average.T
	data_average.to_csv(inOutPrefix+"_average_frame.txt",sep="\t",index=0)
	return data_average
def main():
	parsed=create_parser_for_hydropathy_plot()
	(options,args)=parsed.parse_args()
	(inputFile,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,left_position,right_position,axvline,mode,ylab,start,window,step)=(options.density_file,options.downstream_codon,options.upstream_codon,
	options.output_prefix,options.output_format,options.ymin,options.ymax,options.left_position,options.right_position,options.axvline,options.mode,options.ylab,options.start_position,options.window,options.step)
	if window%2 == 0:
		raise IOError("Please reset your --window parameter. It must be a odd number.")
	if (start-1) < (window-1)/2:
		raise IOError("Please reset your --step and --window parameters. The (window-1)/2 must be less than start-1")
	data=pd.read_csv(inputFile,sep="\t")
	samples=np.unique(data.iloc[:,0])
	text_font={"size":30,"family":"Arial","weight":"bold"}
	legend_font={"size":30,"family":"Arial","weight":"bold"}
	data_average=slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,start,window,step)
	if mode == 'all':
		if (not left_position and right_position) or (left_position and not right_position):
			raise IOError("Do you want to plot in a specific region? Input both the left and right position please!")
		elif ((not left_position) and (not right_position)):
			plot_all_density(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_all_density(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_all_density(data_average,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_all_density(data_average,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			print("finished plot the hydropathy or charge density",file=sys.stderr)
		else:
			if left_position>=right_position:
				raise IOError("The left position must less than the right position. Please reset you -l and -r parameters!")
			if right_position>abs((in_extendRegionLengthParma-in_regionLengthParma)):
				raise IOError("The right position is out of region. Please reset your -r parameer!")
			plot_density_for_specific_region(data,samples,"start codon",left_position,right_position,output_prefix,output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_density_for_specific_region(data,samples,"stop codon",left_position,right_position,output_prefix,output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_density_for_specific_region(data_average,samples,"start codon",left_position,right_position,output_prefix+"_average",output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_density_for_specific_region(data_average,samples,"stop codon",left_position,right_position,output_prefix+"_average",output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			print("finished plot the hydropathy or charge density",file=sys.stderr)
	elif mode == 'single':
		if (not left_position and right_position) or (left_position and not right_position):
			raise IOError("Do you want to plot in a specific region? Input both the left and right position please!")
		elif ((not left_position) and (not right_position)):
			plot_density_for_each_sample(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_density_for_each_sample(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,ymin,ymax,axvline,ylab=ylab,text_font=text_font,legend_font=legend_font)
			print("finished plot the hydropathy or charge density",file=sys.stderr)
		else:
			if left_position>=right_position:
				raise IOError("The left position must less than the right position. Please reset you -l and -r parameters!")
			if right_position>abs((in_extendRegionLengthParma-in_regionLengthParma)):
				raise IOError("The right position is out of region. Please reset your -r parameer!")
			plot_density_for_specific_region(data,samples,"start codon",left_position,right_position,output_prefix,output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			plot_density_for_specific_region(data,samples,"stop codon",left_position,right_position,output_prefix,output_format,ymin,ymax,axvline,mode,ylab=ylab,text_font=text_font,legend_font=legend_font)
			print("finished plot the hydropathy or charge density",file=sys.stderr)
	else:
		raise IOError("Please enter a correct --mode parameter![all or single]")

if __name__ =="__main__":
	main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
	usage: MetageneAnalysisForTheWholeRegions -i 1.bam,2.bam -o test -c longest.trans.info.txt -r 28,29,30_27,28,29 -s 12,12,12_12,12,12 -t bamFile1,bamFile2 -b 15,90,60
				-l 150 (codon) -m 1 -n 10 -e 30 -S selective_transcripts.txt --id-type transcript_id(default) --plot yes
		or MetageneAnalysisForTheWholeRegions -f bamlistFile_attributes.txt -o test -c longest.trans.info.txt -b 15,90,60
				-l 150 (codon) -m 1 -n 10 -e 30 -S selective_transcripts.txt --id-type transcript_id(default) --plot yes
		input:
			1) bamFiles
			2) coordinate file (generated by generated by OutputTranscriptInfo.py)
			3) read length of reads
			4) offset of reads you selected
			5) bamFile legends
			6) bins you want to set, separated by comma, representing relative length of 5UTR, CDS, and 3UTR
			7) et, al.
		output:
			1) metagene results. dataframe format, total bins rows  and N columns. (total bins = 5UTR+CDS+3UTR, N = number of samples)
'''



from __future__ import division
from .FunctionDefinition import *
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns





def scale_transcripts_length(read_counts_vector,bin_number,Type=None):
    if Type == '5UTR': ## do not contain start codon
        bins=int(bin_number)
        steps=int(len(read_counts_vector)/bins)
        read_counts_vector_scaled=np.zeros(bins,dtype="float64")
        if len(read_counts_vector) <= bins:
            if len(read_counts_vector) == 0:
                read_counts_vector=read_counts_vector_scaled
            read_counts_vector_scaled[-len(read_counts_vector):]+=read_counts_vector
        else:
            tmp_read_counts_vector_scaled=[np.mean(read_counts_vector[i:(i+steps)]) for i in np.arange(0,len(read_counts_vector),steps)]
            read_counts_vector_scaled[:]+=tmp_read_counts_vector_scaled[:bins]
            read_counts_vector_scaled[-1]=np.mean(tmp_read_counts_vector_scaled[(bins-1):])
    if Type == "CDS":
        bins=int(bin_number)
        steps=int(len(read_counts_vector)/bins)
        read_counts_vector_scaled=np.zeros(bins,dtype="float64")
        if len(read_counts_vector) <= bins:
            raise IOError("The length of this script is less than 90nt")
        else:
            tmp_read_counts_vector_scaled=[np.mean(read_counts_vector[i:(i+steps)]) for i in np.arange(0,len(read_counts_vector),steps)]
            read_counts_vector_scaled[:]+=tmp_read_counts_vector_scaled[:bins]
            read_counts_vector_scaled[-1]=np.mean(tmp_read_counts_vector_scaled[(bins-1):])
            read_counts_vector_scaled=read_counts_vector_scaled/np.mean(read_counts_vector_scaled)
    if Type == "3UTR": ## do not contain stop codon
        bins=int(bin_number)
        steps=int(len(read_counts_vector)/bins)
        read_counts_vector_scaled=np.zeros(bins,dtype="float64")
        if len(read_counts_vector) <= bins:
            if len(read_counts_vector) == 0:
                read_counts_vector=read_counts_vector_scaled
            read_counts_vector_scaled[:len(read_counts_vector)]+=read_counts_vector
        else:
            tmp_read_counts_vector_scaled=[np.mean(read_counts_vector[i:(i+steps)]) for i in np.arange(0,len(read_counts_vector),steps)]
            read_counts_vector_scaled[:]+=tmp_read_counts_vector_scaled[:bins]
            read_counts_vector_scaled[-1]=np.mean(tmp_read_counts_vector_scaled[(bins-1):])
    return read_counts_vector_scaled

def NormedDensityCalculation(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,inCDS_lengthFilterParma,inCDS_countsFilterParma,in_excludeLengthParma,in_excludeCodonCountsParma,in_readLengths,in_readOffset,Bins):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	passTransSet=set()
	Bins_vector=lengths_offsets_split(Bins)
	final_density_vector=np.zeros(np.sum(Bins_vector),dtype="float64")
	trans_density_scaled=[]
	filter_1=0
	filter_2=0
	filter_3=0
	filter_4=0
	filter_5=0
	all_counts=0
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads ## total_reads for transcript level
	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1# the first position of start codon
		rightCoor=int(in_stopCodonCoorDict[trans])-3 ## the first position of stop codon
		if ((rightCoor-leftCoor) < inCDS_lengthFilterParma*3): # codon level
			filter_1+=1
			continue
		if (rightCoor-leftCoor)%3 !=0:
			filter_2+=1
			continue
		# get transNum and trans 5Pos offset counts vector
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		if total_reads==0 or in_transLengthDict[trans]==0:
			filter_3+=1
			continue
		trans_counts_density=10**6*(trans_counts/(all_counts))
		cds_reads_counts_density=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)))
		if cds_reads_counts_density < inCDS_countsFilterParma: ## RPKM criteria
			filter_4+=1
			continue
		normValue=np.mean(trans_counts_density)
		sumValue=np.sum(read_counts_frameSum[int(in_excludeLengthParma):])
		sumValue_normed=10**9*(sumValue/(all_counts*len(read_counts_frameSum)))
		if normValue == 0 or sumValue_normed < in_excludeCodonCountsParma: ## RPKM criteria
			filter_5+=1
			continue
		trans_counts_density_normed=trans_counts_density/normValue
		Five_UTR_density=trans_counts_density_normed[:leftCoor] ## do not contain start codon
		cds_reads_density=trans_counts_density_normed[leftCoor:(rightCoor+3)] ## contain start codon and stop codon
		Three_UTR_density=trans_counts_density_normed[(rightCoor+3):] ## do not contain stop codon
		# print(trans,len(Five_UTR_density),len(cds_reads_density),len(Three_UTR_density))
		Five_UTR_density_scaled=scale_transcripts_length(Five_UTR_density,Bins_vector[0],Type="5UTR")
		cds_reads_density_scaled=scale_transcripts_length(cds_reads_density,Bins_vector[1],Type="CDS")
		Three_UTR_density_scaled=scale_transcripts_length(Three_UTR_density,Bins_vector[2],Type="3UTR")
		tmp_trans_reads_density_scaled=list(Five_UTR_density_scaled)+list(cds_reads_density_scaled)+list(Three_UTR_density_scaled)
		trans_density_scaled.append(tmp_trans_reads_density_scaled)
		passTransSet.add(trans)
	trans_density_scaled=np.array(trans_density_scaled)
	# print(trans_density_scaled.shape)
	for terms in range(np.sum(Bins_vector)):
		final_density_vector[terms]=np.mean(trans_density_scaled[:,terms])
	pysamFile.close()
	print("Lenght filter(-l)---Transcripts number filtered by criterion one is : "+str(filter_1),file=sys.stderr)
	print("Lenght filter (3n)---Transcripts number filtered by criterion two is : "+str(filter_2),file=sys.stderr)
	print("Total counts filter---Transcripts number filtered by criterion three is : "+str(filter_3),file=sys.stderr)
	print("CDS density filter(RPKM-n or counts-n)---Transcripts number filtered by criterion four is : "+str(filter_4),file=sys.stderr)
	print("CDS density filter(normed-m)---Transcripts number filtered by criterion five is : "+str(filter_5),file=sys.stderr)
	print("Metaplots Transcript Number for bam file"+in_bamFile+" is :"+str(len(passTransSet)),file=sys.stderr)
	return final_density_vector

def MetagenePLotForTheWholeRegions(data,bins,inOutPrefix):
	'''plot the density dsitribution'''
	samples=np.unique(data.columns)
	plt.rc('font',weight='bold')
	text_font={"size":20,"family":"Arial","weight":"bold"}
	legend_font={"size":20,"family":"Arial","weight":"bold"}
	fig=plt.figure(figsize=(10,6))
	ax=fig.add_subplot(111)
	bins_vector=lengths_offsets_split(bins)
	winLen=np.sum(bins_vector)
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	for i in np.arange(len(samples)):
		plt.plot(np.arange(0,winLen),data.loc[:,samples[i]],color=colors[i],label=samples[i],linewidth=2)
	ax.set_xticks([])
	ax.axvline(int(bins_vector[0]),color='gray',dashes=(2,3),clip_on=False,linewidth=2)
	ax.axvline(int(bins_vector[0])+int(bins_vector[1]),color='gray',dashes=(2,3),clip_on=False,linewidth=2)
	ax.set_ylabel("Relative footprint density (AU)",fontdict=text_font)
	ax.set_xlabel("Normalized transcript length",fontdict=text_font,labelpad=30)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2)
	plt.legend(loc="best",prop=legend_font)
	plt.tight_layout()
	# plt.subplots_adjust(left=0.125,right=0.9,bottom=0.15,top=0.9)
	plt.savefig(inOutPrefix+"_metagenePlot_forTheWholeRegions.pdf")
	plt.close()

def write_scaled_density_dataframe(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		d=bms.final_density_vector
		i=bms.bamLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t",index=None)
	return data

def main():
	parsed=create_parser_for_metagene_analysis_for_the_whole_regions()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	if len(options.bins.strip().split(',')) !=3:
		raise IOError("Please check your -b parameters! It must have three numbers separated by comma. e.g. '15,90,75'")
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	for bamfs in bam_attr:
		(bamfs.final_density_vector) = NormedDensityCalculation(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,
        stopCodonCoorDict,options.min_cds_codon,options.min_cds_counts,options.norm_exclude_codon,options.min_norm_region_counts,bamfs.bamLen,bamfs.bamOffset,options.bins)
		print("Finish the step of read density calculation!",file=sys.stderr)
	## write density
	data=write_scaled_density_dataframe(bam_attr,options.output_prefix+"_scaled_density_dataframe.txt")
	print("Finish the step of MetageneAnalysisForTheWholeRegions!",file=sys.stderr)
	if options.plot.upper() in ['YES','Y']:
		MetagenePLotForTheWholeRegions(data,options.bins,options.output_prefix)
		print("Finish the step of plotting!",file=sys.stderr)
	elif options.plot.upper() in ['NO','N','NONE']:
		pass
	else:
		raise IOError("Please input a correct --plot parameter! [yes/no]")

if __name__=="__main__":
    main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-16 08:51:14
@LastEditors: Li Fajin
@LastEditTime: 2019-11-21 17:32:33
@Description: This script is used for calculating ribosome density for each different reading frame.
'''


from .FunctionDefinition import *
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns


def get_ribo_density_of_different_frame(ribo_fileobj, transcript_name, read_lengths, read_offsets, transLength, startCoor, stopCoor):
	"""For each mapped read of the given transcript in the BAM file,get the P-site and codon unit reads density
	ribo_fileobj -- file object - BAM file opened using pysam AlignmentFile
	transcript_name -- Name of transcript to get counts for
	read_length -- If provided, get counts only for reads of this length.
	read_offsets -- the offset length corresponding to 5' mapped position.
	transLength -- the length of the transcript.
	startCoor -- the coordinate of the first base of start codon 0-based.
	stopCoor -- the coordinate of the first base of stop codon 0-based.
	"""
	read_counts = np.zeros(transLength,dtype="int64")
	total_reads = 0
	if read_lengths == "ALL" : ## RNA
		for record in ribo_fileobj.fetch(transcript_name):
			if record.flag == 16 or record.flag == 272:
				continue
			total_reads += 1
			position = record.pos
			read_counts[position]+=1
	else:
		read_lengths=lengths_offsets_split(read_lengths)
		read_offsets=lengths_offsets_split(read_offsets)
		for record in ribo_fileobj.fetch(transcript_name):
			if record.flag == 16 or record.flag == 272:
				continue
			for R_length, R_offset in zip(read_lengths,read_offsets):
				if  record.query_length == R_length :
					# if an offset is specified, increment position by that offset.
					position = record.pos + R_offset ## transform into the position of P-site
				else:
					# ignore other reads/lengths
					continue
				total_reads += 1
				try:
					read_counts[position]+=1
				except KeyError:
					print("Dont has this position after offset : transcript_name -> position"+" "+transcript_name+" -> "+position)
	#get trans counts for each 3 frames
	read_counts_frame0=read_counts[(startCoor+0):(stopCoor-2):3]
	read_counts_frame1=read_counts[(startCoor+1):(stopCoor-1):3]
	read_counts_frame2=read_counts[(startCoor+2):(stopCoor-0):3]
	frame0=np.sum(read_counts_frame0)
	frame1=np.sum(read_counts_frame1)
	frame2=np.sum(read_counts_frame2)
	frameSum=np.sum(frame0+frame1+frame2)
	return frame0,frame1,frame2,frameSum

def Output_frame_density(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,output_prefix):
	'''
	output density on different frames.
	echo row means each transcripts and four columns represents frame0,frame1,frame2 and framSum.
	'''
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans).intersection(in_startCodonCoorDict.keys())
	frame_density=defaultdict(int)
	with open(output_prefix+"_reading_frames.txt",'w') as f:
		f.write("%s\t%s\t%s\t%s\t%s\n" %("transcript_id","frame0","frame1","frame2","frameSum"))
		for trans in in_selectTrans:
			leftCoor =int(in_startCodonCoorDict[trans])-1
			rightCoor=int(in_stopCodonCoorDict[trans])-3
			frame0,frame1,frame2,frameSum=get_ribo_density_of_different_frame(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			f.write("%s\t%s\t%s\t%s\t%s\n" %(trans,str(frame0),str(frame1),str(frame2),str(frameSum)))
			frame_density['frame0']+=frame0
			frame_density['frame1']+=frame1
			frame_density['frame2']+=frame2

	return frame_density

def plot_bar(frame_density,output_prefix):
	''' plot density from different frame'''
	text_font={"size":20,"family":"Arial","weight":"bold"}
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	colors=sns.color_palette('husl',len(frame_density))
	plt.bar(frame_density.keys(),frame_density.values(),color=colors)
	ax.set_ylabel("Raw counts",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,length=2)
	plt.tight_layout()
	plt.savefig(output_prefix+"_reading_frames.pdf")
	plt.close()

def parse_args_for_frame_density():
	parsed=create_parser_for_frame_density()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans

	for bamfs in bam_attr:
		(bamfs.frame_density) = Output_frame_density(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,options.output_prefix+"_"+bamfs.bamLegend)
		if options.plot.upper() in ['YES','Y']:
			plot_bar(bamfs.frame_density,options.output_prefix+"_"+bamfs.bamLegend)
			print("Finish the step of plotting!",file=sys.stderr)
		elif options.plot.upper() in ['NO','N','NONE']:
			pass
		else:
			raise IOError("Please input a correct --plot parameter! [yes/no]")

	print("Finish the step of RiboDensityOfDiffFrames!",file=sys.stderr)



def main():
	"""main program"""
	## parse the options and args and output
	parse_args_for_frame_density()


if __name__ == "__main__":
		main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@since: 2019-08-12 09:57:47
@lastTime: 2019-08-12 15:49:18
@LastAuthor: Li Fajin
@Description:
	This script is used for extracting the longest and all transcript of each gene and output its
	annotation information.There are four inputs:
	1) coorFile: generated by RiboCode --> transcripts_cds.txt
	2) transcriptFile: generated by RiboCode --> transcripts_sequences.txt
	3) gtfFile: annotation file with gtf format for specific species
	4) TransInfoFile: the name of output file
	Usage: python OutputTranscriptInfo.py -c transcripts_cds.txt -g gtfFile.gtf -f transcripts_sequences.fa -o longest.transcripts.info.txt -O all.transcripts.info.txt
'''


from .FunctionDefinition import *
from itertools import groupby




def main():
    parser=create_parser_for_output_transInfo()
    (options,args)=parser.parse_args()
    if options.longestTransInfo and not options.allTransInfo:
        print("Starting outputing longest trans...")
        get_longest_transcripts_information(options.coorFile,options.transcriptFile,options.gtfFile,options.longestTransInfo)
        print("Finishing!")
    elif options.allTransInfo and not options.longestTransInfo:
        print("Starting outputing all trans...")
        get_all_transcripts_information(options.coorFile,options.transcriptFile,options.gtfFile,options.allTransInfo)
        print("Finishing!")
    elif options.longestTransInfo and options.allTransInfo:
        print("Starting outputing longest trans...")
        get_longest_transcripts_information(options.coorFile,options.transcriptFile,options.gtfFile,options.longestTransInfo)
        print("Finishing!")
        print("Starting outputing all trans...")
        get_all_transcripts_information(options.coorFile,options.transcriptFile,options.gtfFile,options.allTransInfo)
        print("Finishing!")
    else:
        raise IOError("Input error! check your -O or -o parameters.")


if __name__=="__main__":
	main()
